#!/usr/bin/env python
import os
from jinjasql import __version__

long_description = '''
Generate SQL Queries using a Jinja Template, without worrying about SQL Injection

JinjaSQL automatically binds parameters that are inserted into the template.
After JinjaSQL evaluates the template, you get 1) Query with placeholders
for parameters, and 2) List of values that need to be bound to the query. 

JinjaSQL doesn't actually execute the query - it only prepares the 
query and the bind parameters. You can execute the query using any 
database engine / driver you are working with.

'''

sdict = {
    'name' : 'jinjasql',
    'version' : __version__,
    'description' : 'Generate SQL Queries and Corresponding Bind Parameters using a Jinja2 Template',
    'long_description' : long_description,
    'url': 'https://github.com/hashedin/jinjasql',
    'download_url' : 'http://cloud.github.com/downloads/hashedin/jinjasql/jinjasql-%s.tar.gz' % __version__,
    'author' : 'Sripathi Krishnan',
    'author_email' : 'Sripathi@hashedin.com',
    'maintainer' : 'Sripathi Krishnan',
    'maintainer_email' : 'Sripathi@hashedin.com',
    'keywords' : ['Jinja2', 'SQL', 'Python', 'Template'],
    'license' : 'MIT',
    'packages' : ['jinjasql'],
    'test_suite' : 'tests.all_tests',
    'classifiers' : [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ]
}

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(**sdict)

